% maximize sum of congruences between AT and B
%
% input: A,B (same size)
%	
% uses nrm.m phi.m tr.m ssq.m ed.m

disp(' Program for Orthogonal Rotation to Maximal sum of Congruences between columns of A and B');
disp(' written by Henk A.L. Kiers, 1995, updated 2005');
disp(' Method described in Kiers & Groenen 1996, Psychometrika, 61, 375-389');
disp('  ')
disp(' You must have your matrices A and B in work space, and they must have the same size');
disp(' ');

disp(' What starting configuration do you want to use for the algorithm?');
start=input(' 1= Rational Procrustes start (=default), 2= Random, 3= Current values: ');
if isempty(start),start=1;end;
disp(' ');

c=input(' What convergence criterion do you want to use (default = 1e-6) ');
if isempty(c), c=1e-6;end;


disp(' ');
disp(' ');
disp(' ');

[n,r]=size(A);
W=A'*nrm(B);		% Step 1
C=A'*A;			% Step 2
[K,L]=ed(C);
rho=L(1,1);		% Step 3
if start==1		% Step 4
  [P,DD,Q]=svd(A'*B);
  T=P*Q';
end;
if start==2
  T=orth(rand(r,r)-.5);
end;
for l=1:r		% Step 4 (second part)
  if W(:,l)'*T(:,l)<0, T(:,l)=-T(:,l); end;	% take care that wl'tl>0
end;
I=eye(r);
f=tr(phi(A*T,B));	% Step 5
fprintf(' Sum of congruences at start is %12.8f;  \n',f);
fold=f-2*c*abs(f);	
iter=0;U=[];
while f-fold>c*f

  Told=T;fold=f;				% Step 6
  p=diag(T'*C*T);		  		% Step 7a
  q=diag(T'*W);	% q(l)=wl'tl			% Step 7b
  for l=1:r		
    if q(l)<0,T(:,l)=-T(:,l);end;		% Step 10
    if q(l)~=0					% Step 7c	
     U(:,l)=q(l)*p(l)^(-1.5)*(C*T(:,l)-rho*T(:,l));
     U(:,l)=U(:,l)-2/p(l)^.5/q(l)*W(:,l)'*W(:,l)*T(:,l);
     U(:,l)=U(:,l)-W(:,l)/p(l)^.5;
   end;
   if q(l)==0,U(:,l)=zeros(r,1);end;
  end;
  [P,DD,Q]=svd(U,0);	% Step 8
  T=-P*Q';		% Step 9
  iter=iter+1;
  f=tr(abs(phi(A*T,B)));	% Step 11
  IT(iter)=f;
end
fprintf(' Sum of congruences after convergence at iteration %g is %12.8f;  \n',iter,f);       
